export declare class SegmentTree {
    /**
     * Input array
     *
     * @type {number[]}
     * @private
     */
    private readonly input;
    /**
     * Operation function
     *
     * @type {function<*>}
     * @private
     */
    private readonly operation;
    /**
     * Fallback value for non-intersect intervals
     *
     * @type {function}
     * @private
     */
    private readonly fallbackValue;
    /**
     * Tree instance
     *
     * @type {number[]}
     */
    private readonly tree;
    /**
     * Creates and builds segment tree
     * Time complexity: O(n)
     *
     * @param {number[]} input
     * @param {function} operation
     * @param {number} fallbackValue
     */
    constructor(input: number[], operation: (...args: any[]) => number, fallbackValue?: number);
    /**
     * Query on segment tree in context of this.operation function
     * Time complexity: O(log(n))
     *
     * @param {number} queryLeftIndex
     * @param {number} queryRightIndex
     * @return {number}
     */
    query(queryLeftIndex?: number, queryRightIndex?: number): number;
    /**
     * Updates element of input array
     * Time complexity: O(log(n))
     *
     * Current version of recursive algorithm
     * is much faster than iterative
     *
     * @param {number} inputIndex
     * @param {number} newValue
     * @param {boolean} recursive
     */
    update(inputIndex: number, newValue: number, recursive?: boolean): void;
    /**
     * @returns {number[]}
     */
    private createTree;
    /**
     * Build segment tree.
     */
    private buildTree;
    /**
     * Build segment tree recursively.
     *
     * @param {number} leftIndex
     * @param {number} rightIndex
     * @param {number} position
     */
    private buildTreeRecursively;
    /**
     * Query on segment tree recursively in context of this.operation function
     *
     * @param {number} queryLeftIndex left index of the query
     * @param {number} queryRightIndex right index of the query
     * @param {number} leftIndex left index of input array segment
     * @param {number} rightIndex right index of input array segment
     * @param {number} position root position in binary tree
     * @return {number}
     */
    private queryRecursive;
    /**
     * @param {number} inputIndex
     * @param {number} newValue
     * @param {number} leftIndex
     * @param {number} rightIndex
     * @param {number} position
     */
    private updateRecursive;
    /**
     * @param {number} inputIndex
     * @param {number} newValue
     * @param {number} leftIndex
     * @param {number} rightIndex
     * @param {number} position
     */
    private updateIterative;
    /**
     * Computes left child index
     *
     * @param {number} position
     * @returns {number}
     */
    private computeLeftChildIndex;
    /**
     * Computes right child index
     *
     * @param {number} position
     * @returns {number}
     */
    private computeRightChildIndex;
}
